Class: Async::GRPC::XDS::ClusterDiscoveryService
- Inherits:
-
DiscoveryService
- Object
- Service
- DiscoveryService
- Async::GRPC::XDS::ClusterDiscoveryService
- Defined in:
- lib/async/grpc/xds/cluster_discovery_service.rb
Overview
Serves Cluster Discovery Service requests from a ControlPlane.
Defined Under Namespace
Classes: Interface
Constant Summary collapse
- SERVICE_NAME =
"envoy.service.cluster.v3.ClusterDiscoveryService"- RESOURCE_TYPE =
ControlPlane::CLUSTER_TYPE
Instance Method Summary collapse
-
#delta_clusters(input, output, call) ⇒ Object
Reject a delta cluster discovery stream, which is not supported.
-
#initialize(control_plane) ⇒ ClusterDiscoveryService
constructor
Initialize a Cluster Discovery Service.
-
#stream_clusters(input, output, call) ⇒ Object
Serve a state-of-the-world cluster discovery stream.
Methods inherited from DiscoveryService
#delta_resources, #stream_resources
Constructor Details
#initialize(control_plane) ⇒ ClusterDiscoveryService
Initialize a Cluster Discovery Service.
35 36 37 |
# File 'lib/async/grpc/xds/cluster_discovery_service.rb', line 35 def initialize(control_plane) super(Interface, SERVICE_NAME, control_plane, resource_type: RESOURCE_TYPE) end |
Instance Method Details
#delta_clusters(input, output, call) ⇒ Object
Reject a delta cluster discovery stream, which is not supported.
50 51 52 |
# File 'lib/async/grpc/xds/cluster_discovery_service.rb', line 50 def delta_clusters(input, output, call) delta_resources end |
#stream_clusters(input, output, call) ⇒ Object
Serve a state-of-the-world cluster discovery stream.
44 45 46 |
# File 'lib/async/grpc/xds/cluster_discovery_service.rb', line 44 def stream_clusters(input, output, call) stream_resources(input, output) end |