Class: Async::GRPC::XDS::EndpointDiscoveryService
- Inherits:
-
DiscoveryService
- Object
- Service
- DiscoveryService
- Async::GRPC::XDS::EndpointDiscoveryService
- Defined in:
- lib/async/grpc/xds/endpoint_discovery_service.rb
Overview
Serves Endpoint Discovery Service requests from a ControlPlane.
Defined Under Namespace
Classes: Interface
Constant Summary collapse
- SERVICE_NAME =
"envoy.service.endpoint.v3.EndpointDiscoveryService"- RESOURCE_TYPE =
ControlPlane::ENDPOINT_TYPE
Instance Method Summary collapse
-
#delta_endpoints(input, output, call) ⇒ Object
Reject a delta endpoint discovery stream, which is not supported.
-
#initialize(control_plane) ⇒ EndpointDiscoveryService
constructor
Initialize an Endpoint Discovery Service.
-
#stream_endpoints(input, output, call) ⇒ Object
Serve a state-of-the-world endpoint discovery stream.
Methods inherited from DiscoveryService
#delta_resources, #stream_resources
Constructor Details
#initialize(control_plane) ⇒ EndpointDiscoveryService
Initialize an Endpoint Discovery Service.
35 36 37 |
# File 'lib/async/grpc/xds/endpoint_discovery_service.rb', line 35 def initialize(control_plane) super(Interface, SERVICE_NAME, control_plane, resource_type: RESOURCE_TYPE) end |
Instance Method Details
#delta_endpoints(input, output, call) ⇒ Object
Reject a delta endpoint discovery stream, which is not supported.
50 51 52 |
# File 'lib/async/grpc/xds/endpoint_discovery_service.rb', line 50 def delta_endpoints(input, output, call) delta_resources end |
#stream_endpoints(input, output, call) ⇒ Object
Serve a state-of-the-world endpoint discovery stream.
44 45 46 |
# File 'lib/async/grpc/xds/endpoint_discovery_service.rb', line 44 def stream_endpoints(input, output, call) stream_resources(input, output) end |