Class: Async::GRPC::XDS::Service

Inherits:
DiscoveryService show all
Defined in:
lib/async/grpc/xds/service.rb

Overview

Serves Envoy Aggregated Discovery Service requests from a ControlPlane.

Constant Summary collapse

SERVICE_NAME =
"envoy.service.discovery.v3.AggregatedDiscoveryService"

Instance Method Summary collapse

Methods inherited from DiscoveryService

#delta_resources, #stream_resources

Constructor Details

#initialize(control_plane) ⇒ Service

Initialize an Aggregated Discovery Service.



19
20
21
# File 'lib/async/grpc/xds/service.rb', line 19

def initialize(control_plane)
	super(Envoy::Service::Discovery::V3::AggregatedDiscoveryService, SERVICE_NAME, control_plane)
end

Instance Method Details

#delta_aggregated_resources(input, output, call) ⇒ Object

Reject a delta Aggregated Discovery Service stream, which is not supported.



37
38
39
# File 'lib/async/grpc/xds/service.rb', line 37

def delta_aggregated_resources(input, output, call)
	delta_resources
end

#stream_aggregated_resources(input, output, call) ⇒ Object

Serve a state-of-the-world Aggregated Discovery Service stream.



28
29
30
# File 'lib/async/grpc/xds/service.rb', line 28

def stream_aggregated_resources(input, output, call)
	stream_resources(input, output)
end